==> symbol-processing-aa-embeddable-2.1.0-1.0.29-sources.jar: org/jetbrains/kotlin/ir/backend/js/transformers/irToJs/JsPolyfills.kt <==
/*
 * Copyright 2010-2021 JetBrains s.r.o. and Kotlin Programming Language contributors.
 * Use of this source code is governed by the Apache 2.0 license that can be found in the license/LICENSE.txt file.
 */

package ksp.org.jetbrains.kotlin.ir.backend.js.transformers.irToJs

import ksp.org.jetbrains.kotlin.ir.backend.js.utils.JsAnnotations
import ksp.org.jetbrains.kotlin.ir.backend.js.utils.hasJsPolyfill
import ksp.org.jetbrains.kotlin.ir.declarations.*
import ksp.org.jetbrains.kotlin.ir.expressions.IrConst
import ksp.org.jetbrains.kotlin.ir.util.getAnnotation
import ksp.org.jetbrains.kotlin.js.backend.ast.JsStatement

class JsPolyfills {
    private val polyfillsPerFile = hashMapOf<IrFile, HashSet<IrDeclaration>>()

    fun registerDeclarationNativeImplementation(file: IrFile, declaration: IrDeclaration) {
        if (!declaration.hasJsPolyfill()) return
        val declarations = polyfillsPerFile[file] ?: hashSetOf()
        declarations.add(declaration)
        polyfillsPerFile[file] = declarations
    }

    fun saveOnlyIntersectionOfNextDeclarationsFor(file: IrFile, declarations: Set<IrDeclaration>) {
        val polyfills = polyfillsPerFile[file] ?: return
        polyfillsPerFile[file] = polyfills.intersect(declarations).toHashSet()
    }

    fun getAllPolyfillsFor(file: IrFile): List<JsStatement> =
        polyfillsPerFile[file].orEmpty().asImplementationList()

    private fun Iterable<IrDeclaration>.asImplementationList() =
        asSequence().asImplementationList()

    private fun Sequence<IrDeclaration>.asImplementationList(): List<JsStatement> {
        return map { it to it.getAnnotation(JsAnnotations.JsPolyfillFqn)!!.getValueArgument(0)!! }
            .distinctBy { (it.second as IrConst).value as String }
            .flatMap { (container, polyfill) -> translateJsCodeIntoStatementList(polyfill, null, container).orEmpty() }
            .toList()
    }
}

==> symbol-processing-aa-embeddable-2.2.0-2.0.2-sources.jar: org/jetbrains/kotlin/ir/backend/js/transformers/irToJs/JsPolyfills.kt <==
/*
 * Copyright 2010-2021 JetBrains s.r.o. and Kotlin Programming Language contributors.
 * Use of this source code is governed by the Apache 2.0 license that can be found in the license/LICENSE.txt file.
 */

package ksp.org.jetbrains.kotlin.ir.backend.js.transformers.irToJs

import ksp.org.jetbrains.kotlin.ir.backend.js.utils.JsAnnotations
import ksp.org.jetbrains.kotlin.ir.backend.js.utils.hasJsPolyfill
import ksp.org.jetbrains.kotlin.ir.declarations.*
import ksp.org.jetbrains.kotlin.ir.expressions.IrConst
import ksp.org.jetbrains.kotlin.ir.util.getAnnotation
import ksp.org.jetbrains.kotlin.js.backend.ast.JsStatement

class JsPolyfills {
    private val polyfillsPerFile = hashMapOf<IrFile, HashSet<IrDeclaration>>()

    fun registerDeclarationNativeImplementation(file: IrFile, declaration: IrDeclaration) {
        if (!declaration.hasJsPolyfill()) return
        val declarations = polyfillsPerFile[file] ?: hashSetOf()
        declarations.add(declaration)
        polyfillsPerFile[file] = declarations
    }

    fun saveOnlyIntersectionOfNextDeclarationsFor(file: IrFile, declarations: Set<IrDeclaration>) {
        val polyfills = polyfillsPerFile[file] ?: return
        polyfillsPerFile[file] = polyfills.intersect(declarations).toHashSet()
    }

    fun getAllPolyfillsFor(file: IrFile): List<JsStatement> =
        polyfillsPerFile[file].orEmpty().asImplementationList()

    private fun Iterable<IrDeclaration>.asImplementationList() =
        asSequence().asImplementationList()

    private fun Sequence<IrDeclaration>.asImplementationList(): List<JsStatement> {
        return map { it to it.getAnnotation(JsAnnotations.JsPolyfillFqn)!!.arguments[0]!! }
            .distinctBy { (it.second as IrConst).value as String }
            .flatMap { (container, polyfill) -> translateJsCodeIntoStatementList(polyfill, container).orEmpty() }
            .toList()
    }
}
